package {
	import com.hurlant.crypto.hash.HMAC;
	import com.hurlant.crypto.hash.SHA256;
	
	import flash.events.MouseEvent;
	import flash.utils.ByteArray;
	
	import mx.collections.Sort;
	import mx.collections.SortField;
	import mx.controls.Alert;
	import mx.core.IMXMLObject;
	import mx.events.FlexEvent;
	import mx.formatters.DateFormatter;
	import mx.rpc.AsyncResponder;
	import mx.rpc.AsyncToken;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.http.HTTPService;
	import mx.utils.Base64Encoder;
	import mx.utils.ObjectProxy;
	
	/**
	 * ロジック部を実装するビューヘルパー
	 * 
	 */
	public class AmazonRankingHelper implements IMXMLObject
	{
		private const awsSecretKey:String = "◆Amazon Product Advertising API の Secret Key◆";
		
		private var _doc:AmazonRanking;
		
		private var httpService:HTTPService
		
		private var parameters:Object;
		
		/**
		 * コンストラクタ
		 */
		public function AmazonRankingHelper() {
			parameters = {
 				"AWSAccessKeyId": "◆Amazon Product Advertising API の AWSAccessKeyId◆",
 				"Operation": "ItemSearch",
 				"ResponseGroup": "Medium",
				"SearchIndex": "Books",
				"Service": "AWSECommerceService",
				"Sort": "salesrank",
				"Version": "2009-01-06"
				};
		}
		
		/**
		 * MXML初期化時の処理
		 */
		public function initialized(document:Object, id:String):void {
			_doc = document as AmazonRanking;
			
			// HTTPサービスを初期化
			httpService = new HTTPService();
			httpService.resultFormat = HTTPService.RESULT_FORMAT_OBJECT;
			
			// MXMLアプリケーション生成完了時のイベントをハンドル
			_doc.addEventListener(FlexEvent.APPLICATION_COMPLETE, applicationCompleteHandler);
		}
		
		/**
		 * MXMLアプリケーション生成完了時の処理
		 */
		private function applicationCompleteHandler(event:FlexEvent):void {
			// 検索ボタンのクリックイベントをハンドル
			_doc.findButton.addEventListener(MouseEvent.CLICK, findButtonOnClickHandler);
		}
		
		/**
		 * 検索ボタンのクリック時のイベント処理
		 *  
		 * @param event マウスイベント
		 * 
		 */
		private function findButtonOnClickHandler(event:MouseEvent):void {
			httpService.url = getRequestAwsURL(_doc.publisherTextInput.text);
			
			var asyncToken:AsyncToken = httpService.send();
			
			asyncToken.addResponder(new AsyncResponder(
				function(event:ResultEvent, token:Object):void {
					// APIからのレスポンスが正常時の処理
					_doc.tileList.dataProvider = event.result.ItemSearchResponse.Items.Item;
				},
				function(event:FaultEvent, token:Object):void {
					// APIからのレスポンスがエラー時の処理
					Alert.show("通信エラー");
				}));
		}
		
		/**
		 * Amazon APIへのリクエストURLを生成します。
		 * 
		 * @param publisher 出版社検索語
		 * @return URL 
		 * 
		 */
		private function getRequestAwsURL(publisher:String):String {
			// マップから key: value それぞれのマップに変換
			var querySet:Array = [];
			for (var key:String in parameters) {
				querySet.push({ "key": key, "value": parameters[key] });
			}
			
			// 入力された出版社を検索語に追加
			querySet.push({ "key": "Publisher", "value": publisher });
			
			// タイムスタンプを追加
			querySet.push({ "key": "Timestamp", "value": nowTimeStamp() });
			
			// key の昇順でソートする
			var sort:Sort = new Sort();
			sort.fields = [ new SortField("key", false, false, false) ];
			sort.sort(querySet);
			
			// key=value の配列に変換する
			var queryList:Array = [];
			for each (var item:Object in querySet) {
				queryList.push(item["key"] + "=" + encodeURIComponent(item["value"]));
			}
			
			// 署名元の文字列を生成
			var strtosign:String = [ "GET", "webservices.amazon.co.jp", "/onca/xml", queryList.join("&") ].join("\n");
			
			// 署名の生成とクエリへの追加
			var signature:String = hmac(awsSecretKey, strtosign)
			queryList.push("Signature=" + encodeURIComponent(signature));
			
			return 'http://webservices.amazon.co.jp/onca/xml?' + queryList.join("&");
		}
		
		/**
		 * HMAC署名を生成します。
		 * 
		 * @param key 秘密鍵
		 * @param data ハッシュ元データ
		 * @return 署名
		 * 
		 */
		private function hmac(key:String, data:String):String {
			var hkey:ByteArray = new ByteArray();
			var hdata:ByteArray = new ByteArray();
			hkey.writeUTFBytes(key);
			hdata.writeUTFBytes(data);
			
			var hmac256:HMAC = new HMAC(new SHA256);
			var ret:ByteArray = hmac256.compute(hkey, hdata);
			var b64enc:Base64Encoder = new Base64Encoder();
            b64enc.encodeBytes(ret, 0, ret.length);
            return b64enc.flush();
		}
		
		/**
		 * 現在時刻のタイムスタンプを取得します。
		 */
		private function nowTimeStamp():String {
			var formatter:DateFormatter = new DateFormatter();
			formatter.formatString = "YYYY-MM-DDTJJ:NN:SSZ";
			return formatter.format( new Date() );
		}

	}
}